from stkhelper.scenarioobject import ScenarioObject
from stkhelper.toolbox import Toolbox

from comtypes.gen import STKObjects

class AreaTarget(ScenarioObject):
    """
    
    An area target object to be added to the scenario. Inherits from
    stkhelper.ScenarioObject.
    
    Parameters:
        guardian(stkhelper.scenario): The scenario for the area target to be
                                    placed into.
        name(str): Name of the area target.
        coordList(list): List of lat/lon pairs to draw the area target
    
    """
    def __init__(self, guardian, name=None, coordList=None, radius=None):
        #TODO Make coordList mandatory
        self.guardian = guardian
        self.name = name
        self.coordinates = coordList
        
        self.root = self.guardian.guardian.root
        
        self.centroid = Toolbox.ComputeCentroid(self.coordinates)

        self.root.BeginUpdate()
        
        if (len(self.coordinates) != 1):
            self.reference = self.root.CurrentScenario.Children.New(STKObjects.eAreaTarget,self.name)
            self.reference = self.reference.QueryInterface(STKObjects.IAgAreaTarget)
            self.reference.AreaType = STKObjects.ePattern
            patterns = self.reference.AreaTypeData
            patterns = patterns.QueryInterface(STKObjects.IAgAreaTypePatternCollection)
            for i in range(len(self.coordinates)):
                patterns.Add(self.coordinates[i][0],
                             self.coordinates[i][1])
            
            self.reference.AutoCentroid = True
        else:
            self.reference = self.root.CurrentScenario.Children.New(STKObjects.eTarget,self.name);
            self.reference = self.reference.QueryInterface(STKObjects.IAgTarget)
            self.reference.Position.AssignGeodetic(coordList[0][0],coordList[0][1],0)
        
        self.root.EndUpdate()
        
    """
    
    Computes and returns access to a given scenario object
    
    Parameters:
        scenarioObject(stkhelper.scenarioObject): Object to compute the access to.
    
    Returns:
        access(list): The list of access times
    
    """
    def GetAccess(self,scenarioObject):
        #TODO Caused error from inherited method. Fix this in scenarioobjects
        self.root.BeginUpdate()
        access = super().GetAccess(scenarioObject)
        self.root.EndUpdate()
        
        return access
    
    """
    
    Sets an elevation access constraint for the area target.
    
    Parameters:
        angle(float): The angle to restrict the access.
    
    """
    def SetElevationConstraint(self, angle):
        self.root.ExecuteCommand("SetConstraint " + \
                                 "*/AreaTarget/" + str(self.name) + \
                                 " ElevationAngle " + str(float(angle)))


from comtypes.gen import STKObjects
from comtypes import COMError

class Scenario:
    def __init__(self, application, name, timePeriod, startTime=None):
        """
        Creates an instance of the scenario.
        Parameters:
            application (STK11 Instance): Running instance of STK 11.
            name (str): Name of the scenario to be created.
            timePeriod (str): The amount of time you want the scenario to run 
            for.
            startTime (str): The start date of the scenario.
                        Examples: "+24hr", "+365days", "+10days".
        """
        
        
        if startTime == None:
            startTime = 'Today'
        else:
            startTime = str(startTime)
        
        self.startTime = startTime
        self.stopTime = timePeriod
        
        self.guardian = application
        self.name = name.replace(' ','_')
        
        application.root.NewScenario(self.name)
        self.reference = application.root.CurrentScenario
        self.reference = self.reference.QueryInterface(STKObjects.IAgScenario)
        try:
            self.reference.SetTimePeriod(str(startTime),str(timePeriod))
        except COMError:
            raise ValueError("Time period not properly formatted")
            
    def SetTimePeriod(self, startTime, stopTime):    
        """
        Sets a new time period for the scenario.
        Parameters:
            elapsedTime (str): The amound of time you want the scenario to run 
            for. Examples: "+24hr", "+365days", "+10days".
        """

        self.startTime = startTime
        self.stopTime = stopTime
        self.reference.SetTimePeriod(str(startTime),str(stopTime))
        
    def Close(self):     
        """
        Closes the scenario.
        """

        self.guardian.root.CloseScenario()



import os
import datetime

class TLE_Manager:
    
    
    """
    
    Class containing only static methods for processing TLE files.
    
    """
    
    @staticmethod
    def ParseTLE(filename):
    
        """
        
        Parses a TLE file of the given file name.
        
        Parameters:
            filename (str): Name of the TLE file.
            
        """
    
        #Create array for return value
        outputArray = []
    
        #Open TLE file
        tle = open(filename,'r')
    
        #Saves first and second lines to variables
        firstLine = tle.readline()
        secondLine = tle.readline()
    
        #Adds line to output array
        outputArray.append(firstLine)
        outputArray.append(secondLine)
    
        #Close tle file
        tle.close()
    
        return outputArray

    @staticmethod
    def GenerateTLE(application, sscNumber):
        
        """
        Retrieves TLE file for a satellite with a given SSC Number
        Parameters:
            application (uiApplication): Application object that holds the scenario.
            sscNumber (str || int): SSC Number of the satellite to retrieve TLE file from.
        
        """
        
        sscNumber = str(sscNumber)
        command = "CreateTLEFile * AGIServer " + os.getcwd() + "\\" + sscNumber + ".tle SSCNumber " + sscNumber
        application.root.ExecuteCommand(command)
        
class Toolbox:

    @staticmethod
    def ComputeCentroid(coordList):
        
        """
        
        Computes the coordinates for the center of an area target.
        
        Parameters:
            parsedLine (str): Line parsed from target list csv.
            
        Returns:
            midPoint (list): List containing name and coordinates of the middle of the area target.
            
        """
        latSum = 0.0
        lonSum = 0.0
        for point in coordList:
            latSum = latSum + float(point[0])
            lonSum = lonSum + float(point[0])
        
        midLat = latSum/float(len(coordList))
        midLon = lonSum/float(len(coordList))
        
        return (midLat,midLon)
    
    @staticmethod
    def CompareTime(time1,time2):       
        """
        Compares if time1 is greater than time2. If it is, it returns true. If
        it is not, it returns false.
        
        Parameters:
            time1 (datetime): Object to test.
            time2 (datetime): Object to compare to time1.
            
            Returns:
                True if time1 is greater than time2. Otherwise, it returns false.
        
        """

        time1Delta = (time1 - datetime.datetime(1970,1,1)).total_seconds()
        time2Delta = (time2 - datetime.datetime(1970,1,1)).total_seconds()
        
        if time1Delta > time2Delta:
            return True
        else:
            return False
    
    @staticmethod    
    def ConvertTime(time1):              
        """
    
        Converts time from STK format to datetime objects.
    
        Parameters:
            time1 (str): The time string from STK.
        
        Returns:
            timeObj1 (datetime): The datetime objects.
        
        """
        monthDict = {"Jan": 1,
                     "Feb": 2,
                     "Mar": 3,
                     "Apr": 4,
                     "May": 5,
                     "Jun": 6,
                     "Jul": 7,
                     "Aug": 8,
                     "Sep": 9,
                     "Oct": 10,
                     "Nov": 11,
                     "Dec": 12}
        
        time1 = str(time1)
        time1 = time1\
        .replace('(','')\
        .replace(')','')\
        .replace("'",'')\
        .replace(',','')
        if time1[0] == 'u':
            time1 = time1[1:]
        
        splitTime1 = time1.split(' ')
        day1 = splitTime1[0]
        month1 = monthDict[splitTime1[1]]
        year1 = splitTime1[2]
        
        clockTime1 = splitTime1[3]
        
        splitClockTime1 = clockTime1.split(':')
        hours1 = splitClockTime1[0]
        minutes1 = splitClockTime1[1]
        seconds1 = splitClockTime1[2].split('.')[0]
        microseconds1 = splitClockTime1[2].split('.')[1]
        
        timeObj1 = datetime.datetime(
                int(year1),
                int(month1),
                int(day1),
                hour=int(hours1),
                minute=int(minutes1),
                second=int(seconds1),
                microsecond=int(microseconds1) * 1000)
        
        return timeObj1
    
    @staticmethod
    def GetTimeDelta(timeArray):
        """
        
        Computes the time difference between to time instances from STK.
        
        Parameters:
            timeArray (list): A list containting two time instances from STK.
            Each instance is in format: 'DD MM YYYY HH:mm:SS.sss'
            
        Returns:
            deltat (float): Time elapsed in seconds between the two instants.
            
        """
        
        timeObj1 = Toolbox.ConvertTime(str(timeArray[0]))
        timeObj2 = Toolbox.ConvertTime(str(timeArray[1]))
        
        timeDiff = timeObj2 - timeObj1
        print(timeDiff.days,timeDiff.seconds, timeDiff.microseconds)
        deltat = float(timeDiff.days*86400.0) + \
        float(timeDiff.seconds) + \
        (0.001*float(timeDiff.microseconds))
                    
        return deltat

    @staticmethod
    def AddAccessArrays(*args):        
        """
    
        For adding access array to a singluar array.
        
        Parameters:
            *args (list): Lists containing the the name and access array in the 
            following format:
                [name, accessArray]
                
        Returns:
            addedArray (list): The array containing all the added access times.
        
        """
        
        addedArray = []
        if len(args) == 1:
            for accessArray in args[1]:
                for line in accessArray[1]:
                    outputLine = [line[0],line[1],accessArray[0]]
                    addedArray.append(outputLine)
                
            return addedArray
        else:    
            for accessArray in args:
                for line in accessArray[1]:
                    outputLine = [line[0],line[1],accessArray[0]]
                    addedArray.append(outputLine)
                
            return addedArray

    @staticmethod
    def SortAllAccess(allAccessArray):
        """
        
        Sorts an array containg access times into chronological order based
        on the start time of the pass.
        
        Parameters:
            allAccessArray (list): List of all access.
        
        Returns:
            outputArray: The sorted access array.
            
        """
        
        outputArray = allAccessArray
        for i in range(len(outputArray)):
            for j in range(0,len(outputArray)-1):
                time1 = outputArray[j] #Full pass array
                time2 = outputArray[j+1] #Full pass array
                time1D = Toolbox.ConvertTime(time1[0]) #Datetime objects of first contact
                time2D = Toolbox.ConvertTime(time2[0]) #Datetime objects of first contact
                
                if Toolbox.CompareTime(time1D,time2D):
                    outputArray[j] = time2
                    outputArray[j+1] = time1
                    
        return outputArray
    
    @staticmethod
    def AccessToCSV(accessArray,outputFilename):
        """
        
        Writes access to a CSV file.
        
        Parameters:
            accessArray (list): List of all access times.
            outputFilename (str): Name of the csv file to be written.
            
        """
        
        outputFile = open(outputFilename,'w')
        for i in range(len(accessArray)):
            for n in range(len(accessArray[i])):
                if n == len(accessArray[i])-1:
                    delimiter = '\n'
                else:
                    delimiter = ','
                outputFile.write(str(accessArray[i][n]) + delimiter)
                
        outputFile.close()


from stkhelper.application import Application
from stkhelper.areatarget import AreaTarget
#from stkhelper.sensor import Sensor
from stkhelper.satellite import Satellite
from stkhelper.scenario import Scenario
from stkhelper.scenarioobject import ScenarioObject
from stkhelper.toolbox import TLE_Manager, Toolbox


from win32api import GetSystemMetrics
from comtypes.client import CreateObject


class Application:
    
    """
    
    Creates the application for STK
    
    """
    def __init__(self):
        self.__uiApplication = CreateObject("STK11.Application")
        self.__uiApplication.Visible = True #Make graphics visible
        self.__uiApplication.UserControl = True #Enable user control
        self.root = self.__uiApplication.Personality2
        
    """
    
    Used to run STK Connect Commands using the root
    
    Parameters:
        connectCommand(str): The Connect Command string to be executed
    
    """
    def Connect(self,connectCommand):
        self.root.ExecuteCommand(connectCommand)
        
    """
    
    Closes the application
    
    """
    def Close(self):
        self.__uiApplication.Close()



from stkhelper.scenarioobject import ScenarioObject
from stkhelper.toolbox import TLE_Manager

from comtypes import COMError
from comtypes.gen import STKObjects

class Satellite(ScenarioObject):
    
    """
    
    A satellite object to be placed into the current scenario.
    
    Parameters:
        guardian(stkhelper.scenario.Scenario): Scenario for the satellite to be
            placed into.
        name(str): Name of the satellite.
        sscNumber(str): SSC Number of the satellite to use.
        startTime(str): Start time of the satellite propogation.
        stopTime(str): Stop time of the satellite propogation.
    
    """
    def __init__(self, guardian, name, sscNumber, startTime=None, stopTime=None):
        super().__init__(guardian,name)
        self.sscNumber = sscNumber
        self.name = name
        self.path = "*/Satellite/" + name
    
        self.root = guardian.guardian.root    
        TLE_Manager.GenerateTLE(self.root, str(sscNumber))
        self.tle = TLE_Manager.ParseTLE(str(sscNumber) + ".tle") 
        
        if startTime == None:
            startTime = self.guardian.reference.StartTime
        if stopTime == None:
            stopTime = self.guardian.reference.StopTime
            
        self.reference = self.root.CurrentScenario.Children.New(STKObjects.eSatellite, name)

        command = 'SetState */Satellite/' + self.name + ' TLE "' + \
                self.tle[0] + '" "' + self.tle[1] + \
                '" TimePeriod "' + \
                startTime + '" "' + \
                stopTime + '"'
        try:                                 
            self.root.ExecuteCommand(command)
        except COMError:
            raise RuntimeError("Failure to add satellite. Check formatting of TLE.")
            
    """
    
    Get access to another stkhelper.scenarioObject.ScenarioObject.
    
    Parameters:
        scenarioObject(stkhelper.scenarioObject.ScenarioObject): Object to compute
            access to.
    
    Returns:
        access: List of all access over the scenario time period.
        
    """
    def GetAccess(self,scenarioObject):
        self.root.BeginUpdate()
        access = super().GetAccess(scenarioObject)
        self.root.EndUpdate()
        
        return access
    
    """
    
    Calculates the power generated by solar panels over a given time period.
    
    Parameters:
        startTime(str): Start time of the calculation.
        endTime(str): Stop time of the calculation.
        timestep(int): Time step to be used (in seconds).
        radius(float): Bounding radius used for the simulation. Should be as 
            small as possible without clipping the model.
        outputPath(str): Absolute path of the file to be saved.
    
    """
    def GetPower(self,startTime,endTime,timestep,radius,outputPath):
        command = 'VO %s SolarPanel Visualization Radius On %f' % (self.path,radius)
        self.root.ExecuteCommand(command)
        command = 'VO %s SolarPanel Compute "%s" "%s" %i Power "%s"' % \
                    (self.path,startTime,endTime,timestep,outputPath)
                    
        self.root.ExecuteCommand(command)
        
    """
    
    Sets the model of the satellite in the scenario to a provided file (.dae + .anc
    or .mdl)
    
    Parameters:
        modelFile(str): Absolute path to the model file to be used.
    
    """
    def SetModel(self,modelFile):
        command = 'VO %s Model File "%s"' % (self.path,modelFile)
        self.root.ExecuteCommand(command)
        
    """
    
    Sets the attitude of the satellite. View full list of attitudes at:
    https://help.agi.com/stk/Subsystems/connectCmds/connectCmds.htm#cmd_SetAttitudeProfile.htm
    
    Parameters:
        profile(str): Attitude profile to be used.
    
    
    """
    def SetAttitude(self,profile):
        #TODO account for offset
        command = 'SetAttitude %s Profile %s Offset 0 0 0' % (self.path,profile)
        self.root.ExecuteCommand(command)


from stkhelper import application, scenario, satellite

class Basic:
    """
    
    Basic test that opens a scenario, adds a satellite, and closes.
    
    """
    def __init__(self):
        app = application.Application()
        scene = scenario.Scenario(app,'Basic','+24hrs')
        sat = satellite.Satellite(scene,'TestSat',25544)
        
        input("Press enter to close test")
        
        scene.Close()
        app.Close()
        


from setuptools import setup, find_packages

info = open('README.md').read()

setup(
      name='stkhelper',
      version='3.0.0',
      description='Uses the STK software to create simulations for satellite testing',
      long_description=info,
      long_description_content_type='text/markdown',
      author='W. Conor McFerren',
      author_email='cnmcferren@gmail.com',
      url="https://github.com/cnmcferren/stkhelper",
      packages=find_packages(),
      install_requires=[
          'comtypes',
          'pypiwin32',
          'datetime']
)



from win32api import GetSystemMetrics
from comtypes.client import CreateObject
from comtypes.gen import STKObjects

class ScenarioObject:
    def __init__(self, guardian, name):
        self.guardian = guardian
        self.name = name
        
    def GetAccess(self,scenarioObject):
        access = self.reference.GetAccessToObject(scenarioObject.reference)
        access.ComputeAccess()
        intervalCollection = access.ComputedAccessIntervalTimes
        
        try:
            computedIntervals = intervalCollection.ToArray(0,-1)
            self.root.EndUpdate()

            return computedIntervals
        
        except Exception:
            self.root.EndUpdate()

            return 0